# frozen_string_literal: true

require 'universal-git-client/http/response_validation'
require 'httparty'
require 'down'
require 'deep_merge'

module UniversalGitClient
  module Http
    class Base
      include HTTParty
      include ResponseValidation

      def initialize(base_url:, download_url: nil, access_token: nil, private_token: nil)
        if access_token && private_token
          raise ArgumentError, 'access_token and private_token params are mutually exclusive'
        end

        @base_url = base_url.chomp('/')
        @download_url = download_url
        @access_token = access_token
        @private_token = private_token
      end

      private

      attr_reader :base_url, :download_url, :access_token, :private_token

      def user_agent
        UniversalGitClient.config.user_agent
      end

      def default_elements_per_page
        UniversalGitClient.config.default_pagination
      end

      def logger
        UniversalGitClient.config.logger
      end

      def log_level
        UniversalGitClient.config.log_level
      end

      def log_format
        UniversalGitClient.config.log_format
      end

      def default_options
        {
          logger: logger,
          log_level: log_level,
          log_format: log_format,
          base_uri: base_url,
          headers: {
            'Accept' => 'application/json',
            'Content-Type' => 'application/json',
            'Authorization' => (access_token ? "Bearer #{access_token}" : nil),
            'Private-Token' => private_token,
            'User-Agent' => user_agent,
          }.compact,
        }
      end

      def down_default_options
        options = default_options[:headers].reject { |k, _v| %w[Accept Content-Type].include?(k) }
        options.reject { |k, _v| %i[base_uri logger log_level log_format].include?(k) }
        # By default Down allows 2 redirections before raising.
        # We're cranking this up as Bitbucket seems to do more redirects
        # when a project has moved.
        options.merge(max_redirects: 5)
      end
    end
  end
end
