# frozen_string_literal: true

require 'json'
require 'jsonapi/serializer'
require 'deep_merge'
require 'uri'
require 'nitlink'

module UniversalGitClient
  module Normalizers
    class Base
      attr_reader :response, :resource

      def initialize(response, resource)
        @response = response
        @resource = resource
      end

      def normalize
        object = body_as_object
        serializer = serializer_for_resource
        serializer_options = {}
        serializer_options.deep_merge!(pagination_options) if object.is_a?(Array)
        serializer.new(object, serializer_options)
      end

      def serializer_for_resource
        self.class.const_get("#{resource}Serializer")
      end

      def body_as_object
        @body_as_object ||= begin
          struct = JSON.parse(response.body, object_class: OpenStruct)
          if !struct.is_a?(Array) && struct.values
            # Bitbucket embeds collections inside values
            struct.values
          else
            struct
          end
        end
      end

      def pagination_options
        {
          meta: {
            pagination: {
              self: current_page_index,
              first: first_page_index,
              prev: prev_page_index,
              next: next_page_index,
              last: last_page_index,
              per_page: per_page_index
            }
          }
        }
      end

      def current_page_index
        response.request.options[:query][:page].to_s
      rescue StandardError
        nil
      end

      def first_page_index
        fetch_rel_value('first')
      end

      def prev_page_index
        fetch_rel_value('prev')
      end

      def next_page_index
        fetch_rel_value('next')
      end

      def last_page_index
        fetch_rel_value('last')
      end

      def per_page_index
        response.request.options[:query][:per_page].to_s
      rescue StandardError
        nil
      end

      private

      def fetch_rel_value(rel)
        Hash[URI.decode_www_form(links.by_rel(rel).target.query)]['page']
      rescue StandardError
        nil
      end

      def links
        @links ||= Nitlink::Parser.new.parse(response)
      end
    end

    class BaseSerializer
      include ::JSONAPI::Serializer
    end
  end
end
